# sb::Admin::Editor
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Editor;
use strict;
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.00';
# 0.00 [2007/09/10] generated
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'editor.html' };
sub REQUIRED_JS  (){ 'script/codepress.js' };
sub TOOLSET_FILE (){ 'default_toolset.cgi' };
my %mToolsDefault = (
	'opt' => '0000000001111111111111111111',
	'set' => "strong:strong\nem:em\np:p\nblockquote:quote\nul:ul\nli:li\np[class=&quot;note&quot;]:cust1\n:cust2\n:cust3",
	'img' => 'cust1',
);
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'temp' => undef,
		'user' => undef,
	);
	$self->SUPER::initialize(%param,@_);
	$self->generate_accessor(keys(%param));
	$self->temp(sb::Template->new($self->load_template('file'=>TEMPLATE)));
	return $self;
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	return $self->display_editor();
}
# --------------------------------------
# // override base class - template parts
sub resource_dir
{
	my $self = shift;
	return $self->conf->value('dir_temp') . $self->lang->language . '/';
}
sub static_path
{
	my $self = shift;
	return $self->conf->value('srv_temp');
}
# --------------------------------------
# // public functions - main routine
sub display_editor
{
	my $self = shift;
	my %param = (
		'id'        => 'entry_body',
		'code'      => 'html',
		'content'   => '',
		'imgselect' => 1,
		'addimage'  => undef,
		@_
	);
	return if (!$self->temp);
	$self->temp->num(0);
	$self->temp->tag('sb_editor_id'=>$param{'id'});
	$self->temp->tag('sb_editor_code'=>$param{'code'});
	$self->temp->tag('sb_editor_content'=>$param{'content'});
	$self->temp->block('sb_editor_main'=>1);
	$self->common_parts($self->temp);
	$self->_toolicons('editor');
	$self->_image_selector() if ($param{'imgselect'});
	if ($param{'addimage'})
	{
		$self->temp->num(0);
		$self->temp->tag('sb_editor_addimage_target'=>$param{'addimage'});
		$self->temp->block('sb_editor_addimage'=>1);
	}
	return $self->temp->output();
}
sub display_option
{
	my $self = shift;
	return if (!$self->temp);
	$self->temp->num(0);
	foreach my $key ('imagelist','imagemax')
	{
		my $value = $self->user->get_option($key);
		next if ($value !~ /^\d+$/);
		$self->temp->tag('sb_editor_' . $key . '_' . $value => 'selected="selected"');
	}
	$self->temp->block('sb_editor_option'=>1);
	$self->_toolicons('option');
	$self->common_parts($self->temp);
	return $self->temp->output();
}
sub update_option
{
	my $self = shift;
	my @opts = split(//,$mToolsDefault{'opt'});
	my @sets = ();
	my $ext = '';
	# --- update tool bar settings
	push(@sets,( $self->cgi->value('editor_toolurl') ne '' ) ? 0 : 1);
	push(@sets,( $self->cgi->value('editor_toolent') ne '' ) ? 0 : 1);
	push(@sets,1); # textarea resizer, it's for backward compatibility
	splice(@opts,0,3); # the first 3 items in options are special tools, so skip them
	for my $i (0 .. $#opts)
	{
		my $label = 'editor_tool' . $i;
		my $elem = sb::Text->new($self->cgi->value($label . '_elem'))->entitized();
		my $attr = sb::Text->new($self->cgi->value($label . '_opt') )->entitized();
		my $icon = sb::Text->new($self->cgi->value($label . '_icon'))->entitized();
		push(@sets,( $self->cgi->value($label) ne '' ) ? 0 : 1);
		$ext .= $self->_encode_text($elem);
		$ext .= '[' . $self->_encode_text($attr) . ']' if ($attr ne '');
		$ext .= ':' . $icon . "\n";
	}
	$self->user->set_option('edit_tool'=>join('',@sets));
	$self->user->ext($ext);
	# --- update image selector settings
	foreach my $key ('imagelist','imagemax')
	{
		my $value = int($self->cgi->value('editor_' . $key));
		$self->user->set_option($key => $value);
	}
	return sb::Data->update($self->user);
}
# --------------------------------------
# // public functions - class method
sub default_tooloption
{
	my $class = shift;
	return $mToolsDefault{$_[0]};
}
sub javascripts
{
	my $class = shift;
	return sb::Config->get->value('srv_temp') . REQUIRED_JS;
}
# --------------------------------------
# // private functions
sub _toolicons
{
	my $self = shift;
	my $mode = shift;
	return if (!$self->temp);
	my @icons = $self->_load_toolset();
	my @opts = split(//,$mToolsDefault{'opt'});
	my @sets = split(/\n/,$mToolsDefault{'set'});
	if ($self->user and $self->user->get_option('edit_tool') ne '')
	{
		@opts = split(//,$self->user->get_option('edit_tool'));
	}
	if ($self->user and $self->user->ext() ne '')
	{
		@sets = split(/\n/,$self->user->ext());
	}
	# the first 3 items in options are special tools, so we should handle them differently.
	my $url = shift(@opts); # link button
	my $ent = shift(@opts); # convert button
	my $hig = shift(@opts); # textarea resize button (ignored)
	my $num = 0;
	foreach my $i (0 .. $#opts)
	{
		my ($elem,$attr,$img);
		next if ($opts[$i] and $mode ne 'option');
		($elem,$img) = split(':',$sets[$i],2);
		$elem = '' if ($elem eq ''); # to avoid $elem is set as undef
		$img = $mToolsDefault{'img'} if ($img eq '');
		if ($elem =~ /^(.*?)\[(.*?)\]$/)
		{
			$elem = $1;
			$attr = ($2 ne '') ? ' ' . $2 : '';
		}
		next if ($elem eq '' and $mode ne 'option');
		$self->temp->num($num);
		if ($mode eq 'option')
		{ # --- option mode
			$attr =~ s/\s*//g;
			$self->temp->tag('sb_editor_tool_check' => ($opts[$i] == 0) ? 'checked="checked"' : '');
			$self->temp->tag('sb_editor_tool_num'   => $num);
			$self->temp->tag('sb_editor_tool_elem'  => $elem);
			$self->temp->tag('sb_editor_tool_opt'   => $attr);
			$self->temp->tag('sb_editor_tool_img'   => $img);
			my $selector = '';
			foreach my $icon ( @icons )
			{
				$selector .= '<option value="' . $icon . '"';
				$selector .= ' selected="selected"' if ($icon eq $img);
				$selector .= '>' . $icon . '</option>';
			}
			$self->temp->tag('sb_editor_toolicon_selector'=>$selector);
			$self->temp->tag('sb_list_class' => ($num % 2) ? 'odd' : 'even');
		}
		else
		{ # --- editor mode
			$self->temp->tag('sb_editor_tools_icon'=>$img);
			$self->temp->tag('sb_editor_tools_pre' =>'&lt;' . $elem . $attr . '&gt;');
			$self->temp->tag('sb_editor_tools_post'=>'&lt;/' . $elem . '&gt;');
			$self->temp->tag('sb_editor_tools_text'=>'&lt;' . $elem . $attr . '&gt;');
		}
		$num++;
	} # end of foreach my $i (0 .. $#options)
	if ($mode eq 'option')
	{ # --- option mode
		$self->temp->num(0);
		$self->temp->tag('sb_editor_toolurl_check' => ($url == 0) ? 'checked="checked"' : '');
		$self->temp->tag('sb_editor_toolent_check' => ($ent == 0) ? 'checked="checked"' : '');
		$self->temp->block('sb_editor_tool_set' => $num);
	}
	else
	{ # --- editor mode
		$self->temp->block('sb_editor_link' => ($url == 0) ? 1 : 0);
		$self->temp->block('sb_editor_convert' => ($ent == 0) ? 1 : 0);
		$self->temp->block('sb_editor_tools' => $num);
	}
}
sub _image_selector
{
	my $self = shift;
	my %param = (
		'num'    => $self->user->get_option('imagemax'),
		'option' => $self->user->get_option('imagelist'),
		@_
	);
	return if ( !$self->temp );
	my $selector = '';
	my $num = 0;
	my @images = sb::Data->load('Image',
		'num'   => $param{'num'},
		'sort'  => 'date',
		'order' => 1,
		'cond'  => {'stat'=>0},
	);
	my $option = ('file','thumb','link','link')[$param{'option'}];
	foreach my $img ( @images )
	{
		my $type = '';
		my $flag = $img->is_image;
		my $text = sb::Text->new($img->get_as_tag('type'=> $flag ? $option : 'link'));
		if ($flag and $option ne 'file' and $img->thumb ne '')
		{
			$type = ($param{'option'} == 1)
				? sb::Language->get->string('[small]')
				: sb::Language->get->string('[link]');
		}
		elsif (!$flag)
		{
			$type = sb->string('[link]');
		}
		if ($flag and $option eq 'link' and $param{'option'} == 3 and $img->thumb ne '')
		{
			my $orig = sb::Text->new($img->get_as_tag('type'=>'file'));
			$selector .= '<option value="' . $orig->entitized() . '">' . $img->name . '</option>' . "\n";
		}
		$selector .= '<option value="' . $text->entitized() . '">' . $img->name . $type . '</option>' . "\n";
		$num++;
		last if ($param{'num'} > 0 and $num == $param{'num'});
	}
	$self->temp->num(0);
	$self->temp->tag('sb_editor_images'=>$selector);
	$self->temp->block('sb_editor_imageselector'=>1);
}
sub _load_toolset
{
	my $self = shift;
	my $list = $self->load_template(
		'dir'  => $self->conf->value('dir_temp'),
		'file' => TOOLSET_FILE
	);
	return split("\n",$list);
}
sub _encode_text
{
	my $self = shift;
	my $text = shift;
	$text =~ s/:/&#58;/g;
	$text =~ s/\[/&#91;/g;
	$text =~ s/\]/&#93;/g;
	$text =~ s/\'/&#39;/g;
	return $text;
}
1;
__END__
