# sb::Config
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Config;
use strict;
use sb::InitParser ();
use sb::Object ();
use base qw( sb::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.11';
# 0.11 [2007/09/29] modified callback in load to consider the default config file / removed _default_set
# 0.10 [2007/09/26] added config_file
# 0.09 [2007/07/13] added save and changed load to load extra config file
# 0.08 [2007/07/13] changed base class and removed static variables
# 0.07 [2007/07/12] added check_key
# 0.06 [2006/07/04] got rid of "return( undef )" and "return undef"
# 0.05 [2006/04/06] changed load to use sb::InitLoader
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2006/03/17] renamed lang_rsrc to lang_resource
# 0.02 [2006/03/04] changed procedure of initialization to have flexibility
# 0.01 [2005/11/14] refactored
# --------------------------------------
# // defines / static variables
# ... nothing ...
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	eval{ require 'config.pl'; };
	$self->SUPER::initialize($self->setting());
	$self->load();
	return $self;
}
# --------------------------------------
# // public functions
sub value
{
	my $self = shift;
	my $key  = shift;
	return if (! defined($key) );
	$self->{$key} = shift if (@_);
	return $self->{$key};
}
sub lang_resource
{
	my $self = shift;
	my $lang = shift || 'en';
	return $self->value('resource_dir') . $lang . $self->value('language_ext');
}
sub config_file
{
	my $self = shift;
	return $self->value('dir_data') . $self->value('file_conf');
}
sub load
{
	my $self = shift;
	my $file = shift || $self->value('init_file');
	return if (!$file);
	return sb::InitParser->new('file'=>$file,'rule'=>\&_check_key)->load($self);
	sub _check_key
	{
		my $key = shift;
		$key = $self->check_key($key);
		return if ($key eq '');
		return if ($key =~ /\W/);
		return if (! defined($self->{$key}));
		return $key;
	}
}
sub save
{
	my $self = shift;
	my $file = shift;
	return if (!$file);
	return sb::InitParser->new('file'=>$file,'rule'=>\&_saved_key)->save($self);
	sub _saved_key
	{
		my $key = shift;
		$key = $self->saved_key($key);
		return if ($key eq '');
		return if ($key =~ /\W/);
		return if (! defined($self->{$key}));
		return $key;
	}
}
# --------------------------------------
# // public functions - declaration
sub saved_key
{
	my $self = shift;
	my $key = shift;
	return $key;
}
sub check_key
{
	my $self = shift;
	my $key = shift;
	return $key;
}
sub verify_values;
sub setting;
1; # end of package
__END__
