// spica javascript libraries - codepress.js
// == written by Takuya Otani <takuya.otani@gmail.com> ===
// == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// == This script is based on CodePress.
// == Copyright (C) 2006 Fernando M.A.d.S. <fermads@gmail.com>
// == CodePress is free software; you can redistribute it and/or modify it under the terms of the LGPL.
// == Read the full licence: http://www.opensource.org/licenses/lgpl-license.php

Spica.Library.require('string','formhandler','cookie','element');
Spica.CodePress = function(textarea)
{
	this.iframe = null;
	this.textarea = textarea;
	this.widthBase = Spica.DOM.getParent(textarea);
	this.resizer = document.createElement('span');
	this.heightDiff = 0;
	this.body = null;
	this.options = textarea.getAttribute('editor:option');
	this.modified = false;
	this.cookie = null;
	this.initialize().edit();
	return this;
};
Spica.CodePress.prototype = {
	initialize : function()
	{
		var self = this;
		var body = document.getElementsByTagName('body')[0];
		self.cookie = new Spica.Cookie({name:body.id,expire:365});
		// --- iframe
		self.iframe = document.createElement('iframe');
		self.iframe.id = [self.textarea.id,'frame'].join('');
		self.textarea.disabled = true;
		self.iframe.className = 'codepress-frame';
		self.iframe.style.visibility = 'hidden';
		self.iframe.style.position = (Spica.Browser.isSafari && !Spica.Browser.isSafari3) ? 'static' : 'absolute';
		self.iframe.frameBorder = 0; // remove IE internal iframe border
		self.initSize();
		Spica.DOM.insertBefore(self.iframe, self.textarea);
		// --- resizer
		self.resizer.className = 'codepress-resizer';
		self.resizer.style.display = 'none';
		Spica.DOM.insertAfter(self.resizer, self.textarea);
		Spica.Event.register(self.resizer,'mousedown',function(evnt) { self.triggerResize(evnt); });
		return self;
	},
	frameLoaded : function()
	{
		var self = this;
		self.body = self.iframe.contentWindow.document.getElementsByTagName('body')[0];
		self.setOptions();
		self.iframe.contentWindow.CodePress.setCode(self.textarea.value);
		self.iframe.contentWindow.CodePress.syntaxHighlight('init');
		self.iframe.style.position = 'static';
		self.iframe.style.visibility = 'visible';
		self.iframe.style.display = 'inline';
		self.textarea.style.display = 'none';
		self.iframe.contentWindow.CodePress.onModified = function() { self.onModified() };
		if (this.widthBase)
			Spica.Event.register(window,'resize',function() { self.updateWidth(); });
		if (self.cookie.get('enable') == 'off')
			self.disableEditor();
	},
	edit : function()
	{
		var self = this;
		if (!self.textarea.disabled) return;
		var language = 'generic';
		for (var lang in Spica.CodePress.languages)
		{
			if (self.options.match('\\b'+lang+'\\b') && Spica.CodePress.languages[lang])
			{
				language = lang;
				break;
			}
		}
		var path = [Spica.Library.path,'codepress/index.html?language=',language].join('');
		Spica.Event.register(self.iframe,'load',function() { self.frameLoaded(); });
		self.iframe.src = [path,'&ts=',(new Date).getTime()].join('');
	},
	setOptions : function()
	{
		var self = this;
		if (self.options.match('autocomplete-on')) self.toggleAutoComplete();
		if (self.options.match('readonly-on')) self.toggleReadOnly();
		if (self.options.match('linenumbers-on')) self.toggleLineNumbers();
		if (self.options.match('resizer-on')) self.toggleResizer();
		if (self.options.match('width-adj-off')) self.widthBase = null;
	},
	toggleAutoComplete : function()
	{
		var self = this;
		self.iframe.contentWindow.CodePress.autocomplete = !self.iframe.contentWindow.CodePress.autocomplete;
	},
	toggleReadOnly : function()
	{
		var self = this;
		self.textarea.readOnly = !self.textarea.readOnly;
		if (self.iframe.style.display != 'none') // prevent exception on FF + iframe with display:none
			self.iframe.contentWindow.CodePress.readOnly(self.textarea.readOnly);
	},
	toggleLineNumbers : function()
	{
		var self = this;
		var name = self.body.className;
		self.body.className = (name == '' || name == 'hide-line-numbers') 
			? 'show-line-numbers'
			: 'hide-line-numbers';
	},
	toggleResizer : function()
	{
		var self = this;
		self.resizer.style.display = (self.resizer.style.display == 'block') ? 'none' : 'block';
	},
	getCode : function()
	{
		var self = this;
		return self.textarea.disabled
			? self.iframe.contentWindow.CodePress.getCode()
			: self.textarea.value;
	},
	setCode : function(code)
	{
		var self = this;
		self.textarea.disabled
			? self.iframe.contentWindow.CodePress.setCode(code)
			: self.textarea.value = code;
	},
	toggleEditor : function()
	{
		var self = this;
		(self.textarea.disabled)
			? self.disableEditor()
			: self.enableEditor();
		return false;
	},
	enableEditor : function(withoutSave)
	{
		var self = this;
		self.textarea.disabled = true;
		self.iframe.style.display = 'inline';
		self.textarea.style.display = 'none';
		if (Spica.Browser.isSafari && !Spica.Browser.isSafari3) return;
		self.setCode(self.textarea.value);
		self.iframe.contentWindow.CodePress.syntaxHighlight('init');
		if (withoutSave) return;
		self.cookie.set('enable','on');
	},
	disableEditor : function(withoutSave)
	{
		var self = this;
		self.iframe.style.display = 'none';
		self.textarea.style.display = 'inline';
		self.textarea.value = self.getCode();
		self.textarea.disabled = false;
		if (withoutSave) return;
		self.cookie.set('enable','off');
	},
	initSize : function()
	{
		var self = this;
		var height = self.cookie.get('height');
		self.textarea.style.overflow = 'hidden'; // set overflow at once to get width/height
		if (!height)
			height = Spica.DOM.getHeight(self.textarea);
		self.setHeight(height);
		var base = self.widthBase || self.textarea;
		self.iframe.style.width = [Spica.DOM.getWidth(base),'px'].join('');
		self.resizer.style.width = [Spica.DOM.getWidth(base),'px'].join('');
		self.textarea.style.overflow = 'auto'; // put overflow back
	},
	setHeight : function(height)
	{
		var self = this;
		if (height < 100) height = 100;
		self.iframe.style.height = [height,'px'].join('');
		self.textarea.style.height = [height,'px'].join('');
	},
	updateWidth : function()
	{
		var self = this;
		if (self.widthBase)
		{
			self.iframe.style.width = [Spica.DOM.getWidth(self.widthBase),'px'].join('');
			self.resizer.style.width = [Spica.DOM.getWidth(self.widthBase),'px'].join('');
		}
	},
	triggerResize : function(evnt)
	{
		var self = this;
		evnt = (evnt) ? evnt : ((window.event) ? event : null);
		var _bgn = evnt.clientY;
		var _mv = function(evt) { self.doResize(evt,_bgn); };
		var _up = function(evt) { self.endResize(evt,_mv,_up); };
		self.heightDiff = parseInt(self.textarea.style.height); // buffer the textarea height
		if (self.textarea.disabled) self.disableEditor(true);
		Spica.Event.register(document,"mousemove",_mv);
		Spica.Event.register(document,"mouseup",_up);
		Spica.Event.stop(evnt);
	},
	doResize : function(evnt,startPos)
	{
		var self = this;
		evnt = (evnt) ? evnt : ((window.event) ? event : null);
		var diff = parseInt(evnt.clientY - startPos);
		self.setHeight(self.heightDiff + diff);
		Spica.Event.stop(evnt);
	},
	endResize : function(evnt,_mv,_up)
	{
		var self = this;
		evnt = (evnt) ? evnt : ((window.event) ? event : null);
		if (self.cookie.get('enable') != 'off') self.enableEditor(true);
		self.cookie.set('height',parseInt(self.textarea.style.height));
		Spica.Event.deregister(document,"mousemove",_mv);
		Spica.Event.deregister(document,"mouseup",_up);
		Spica.Event.stop(evnt);
	},
	insertCode : function(text)
	{
		var self = this;
		if (!text) return false;
		if (self.textarea.disabled)
		{
			var body = self.body.getElementsByTagName('pre')[0];
			if (!body) body = self.body;
			self.iframe.contentWindow.CodePress.insertCode(text);
			self.iframe.contentWindow.CodePress.syntaxHighlight('generic');
			body.focus();
		}
		else
		{
			var selected = new Spica.SelectedText(self.textarea);
			selected.set_text(text);
		}
		return self.onModified();
	},
	cliponCode : function(before,after)
	{
		var self = this;
		if (self.textarea.disabled)
		{
			var selected = self.iframe.contentWindow.CodePress.getSelectedText();
			self.insertCode([before,selected,after].join(''));
		}
		else
		{
			var selected = new Spica.SelectedText(self.textarea);
			selected.clipon(before,after);
		}
		return self.onModified();
	},
	entitizeCode : function()
	{
		var self = this;
		var selected = (self.textarea.disabled)
			? self.iframe.contentWindow.CodePress.getSelectedText()
			: (new Spica.SelectedText(self.textarea)).get_text();
		return self.insertCode(selected.entitize());
	},
	detitizeCode : function()
	{
		var self = this;
		var selected = (self.textarea.disabled)
			? self.iframe.contentWindow.CodePress.getSelectedText()
			: (new Spica.SelectedText(self.textarea)).get_text();
		return self.insertCode(selected.detitize());
	},
	insertLink : function()
	{
		var self = this;
		var url = prompt(Spica.$_('Enter site address to belinked.'), 'http://');
		if (!url || url == 'http://') return false;
		var target = prompt(Spica.$_('Specify target attribute.'), '_blank');
		if (target == null) return false;
		var title = prompt(Spica.$_('Specify title attribute.'),'');
		if (title == null) return false;
		var link = ['<a href="',url,'"'].join('');
		if (target) link = [link,' target="',target,'"'].join('');
		if (title) link = [link,' title="',title,'"'].join('');
		return self.cliponCode([link,'>'].join(''),'</a>');
	},
	onModified : function()
	{
		var self = this;
		if (self.modified) return false;
		self.modified = true;
		Spica.FormHandler.instance.setConfirmationToLeave(true);
		return false;
	},
	onSubmit : function()
	{
		var self = this;
		if (self.textarea.disabled) self.disableEditor(true);
	}
};
Spica.CodePress.languages = {
	css: 'CSS', 
	generic: 'Generic',
	html: 'HTML',
	javascript: 'JavaScript', 
	perl: 'Perl', 
	php: 'PHP', 
	text: 'Text',
	sbtext: 'sbTextFormat'
};
Spica.CodePress.instance_of = {};
Spica.CodePress.of = function(id)
{
	return Spica.CodePress.instance_of[id];
};
Spica.CodePress.doCopy = function(text)
{ // original code by ma.la http://la.ma.la/
	var swf_data = [Spica.Library.path,'codepress/setClipboard.swf'].join('');
	var buffer = document.createElement("div");
	buffer.innerHTML = [
		 '<embed src="', swf_data, '"'
		,' FlashVars="code=',encodeURI(text),'"'
		,' width="0" height="0"></embed>'
	].join('');
	buffer.style.position ="absolute";
	buffer.style.left = "-10px";
	buffer.style.top  = "-10px";
	buffer.style.visibility = "hidden";
	document.body.appendChild(buffer);
	setTimeout(function(){document.body.removeChild(buffer)},1000)
};
Spica.CodePress.getSubmitter = function(id)
{
	return function() { Spica.CodePress.instance_of[id].onSubmit() };
};
Spica.Event.run(function() {
	var forms = document.getElementsByTagName('form');
	for (var i=0,n=forms.length;i<n;i++)
	{
		var area = forms[i].getElementsByTagName('textarea');
		for (var j=0,m=area.length;j<m;j++)
		{
			if (area[j].className.indexOf('codepress') == -1) continue;
			var id = area[j].id;
			if (!id) id = ['cp_',j].join('');
			if (Spica.CodePress.instance_of[id])
			{ // it has been duplicated, so we should change the id.
				id = [id,'_'].join('');
				area[j].id = id;
			}
			Spica.CodePress.instance_of[id] = new Spica.CodePress(area[j]);
			var handler = Spica.CodePress.getSubmitter(id);
			Spica.Event.register(forms[i],'submit',handler);
		}
	}
});
